/*
 * ipmilanplus.c
 * 
 * Interface to call libintf_lanplus from ipmitool to do RMCP+ protocol.
 *
 */
#include <stdio.h>
#include <stdarg.h>


#ifndef HAVE_LANPLUS
/* No lanplus, so stub these functions returning errors. */
#define uchar  unsigned char
#define ushort  unsigned short
#define LAN_ERR_INVPARAM  -2

int ipmi_open_lan2(char *node, char *user, char *pswd, int fdebugcmd)
{
    return(LAN_ERR_INVPARAM);
}

int ipmi_close_lan2(char *node)
{
    return(LAN_ERR_INVPARAM);
}

int ipmi_cmd_lan2(char *node, ushort cmd, uchar *pdata, uchar sdata,
                uchar *presp, int *sresp, uchar *pcc, char fdebugcmd)
{
    return(LAN_ERR_INVPARAM);
}

#else

#include "ipmilanplus.h"
#include "ipmicmd.h"

extern int gshutdown;    /* from ipmicmd.c */
extern int gauth_type;   /* from ipmicmd.c */
extern int gpriv_level;  /* from ipmicmd.c */
extern FILE *fpdbg;      /* from ipmicmd.c */
extern FILE *fperr;      /* from ipmicmd.c */
extern ipmi_cmd_t ipmi_cmds[]; /* from ipmicmd.c */
extern struct ipmi_intf ipmi_lanplus_intf;  /*from libintf_lanplus.a*/

int verbose = 0;  
static int loglevel = 4;  /*LOG_ERR*/
static int lan2_fd = 0;
static struct ipmi_intf *intf = NULL;

#define LAN_ERR_INVPARAM  -2
#define PSWD_MAX       16
#define LOG_MSG_LENGTH          1024 

struct ipmi_intf * ipmi_intf_load(char * name)
{
    if (strcmp(name,"lanplus") == 0) {
#ifdef WIN32
         /* initialize the intf */
         strcpy(ipmi_lanplus_intf.name,"lanplus");
         ipmi_lanplus_intf.setup =      ipmi_lanplus_setup;
         ipmi_lanplus_intf.open =       ipmi_lanplus_open;
         ipmi_lanplus_intf.close =      ipmi_lanplus_close;
         ipmi_lanplus_intf.sendrecv =   ipmi_lanplus_send_ipmi_cmd;
         ipmi_lanplus_intf.recv_sol =   ipmi_lanplus_recv_sol;
         ipmi_lanplus_intf.send_sol =   ipmi_lanplus_send_sol;
         ipmi_lanplus_intf.keepalive =  ipmi_lanplus_keepalive;
         ipmi_lanplus_intf.target_addr = IPMI_BMC_SLAVE_ADDR;  /*0x20*/
#endif
         return (&ipmi_lanplus_intf);
    }
    else return (NULL);
}

/* 
 * ipmi_open_lan2
 */
int ipmi_open_lan2(char *node, char *user, char *pswd, int fdebugcmd)
{
   // char *username;
   // uchar priv_level;
   // char *temp;
   int rv = -1;
   int n;

   if (fdebugcmd) 
      printf("ipmi_open_lan2(%s,%s,%s,%d)\n",node,user,pswd,fdebugcmd);
   if (fdebugcmd) loglevel = 7;  /*LOG_DEBUG; max=8 (LOG_DEBUG+1)*/
   verbose = fdebugcmd;          /* 0 or 1 */
   if (nodeislocal(node)) {
        fprintf(fpdbg,"ipmi_open_lan2: node %s is local!\n",node);
        rv = LAN_ERR_INVPARAM;
        goto EXIT;
   } else {
        if ((gshutdown==0) || fdebugcmd) 
	   fprintf(fpdbg,"Opening connection to node %s ...\n",node);

        rv = 0;
        if (intf == NULL) {
            /* fill in the intf structure */
            intf = ipmi_intf_load("lanplus");
	    if (intf == NULL) return(-1);
	    if (intf->setup == NULL) return(-1);

            rv = intf->setup(intf);  /*allocates session struct*/
            if (fdebugcmd)
                printf("lan2 intf setup returned %d\n",rv);
        } 

        if (rv == 0) {
	    if (intf->open == NULL) return(-1);
            if (intf->session == NULL) return(-1);
            intf->session->authtype_set = gauth_type; 
	    intf->session->privlvl      = gpriv_level;
            if (node != NULL) {
                strcpy(intf->session->hostname,node);
            }
            if (user != NULL) {
                strcpy(intf->session->username,user);
            }
            if (pswd == NULL || pswd[0] == 0) 
	      intf->session->password = 0;
	    else {
	      intf->session->password = 1;
              n = strlen(pswd);
              if (n > PSWD_MAX) n = PSWD_MAX;
              memset(intf->session->authcode,0,PSWD_MAX);
              strncpy(intf->session->authcode, pswd, n);
            }
            lan2_fd = intf->open(intf);
            if (fdebugcmd)
                printf("lan2 intf open returned %d\n",lan2_fd);
            if (lan2_fd > 0) rv = 0;
            else rv = -1;
        }
   }
EXIT:
   if (rv != 0) {
      if ((gshutdown==0) || fdebugcmd) 
             fprintf(fperr, "ipmi_open_lan2 error %d\n",rv);
   }
   return(rv);
}

int ipmi_close_lan2(char *node)
{
   int rv = 0;

   if (!nodeislocal(node)) {  /* ipmilan, need to close & cleanup */
        /* do the close */
        if (intf->opened > 0 && intf->close != NULL)
             intf->close(intf);
        /* ipmi_cleanup(intf);  * does ipmi_sdr_list_empty(intf) */
   } 
   return (rv);
}
 
/* 
 * ipmi_cmd_lan2
 * This is the entry point, called from ipmicmd.c
 */
int ipmi_cmd_lan2(char *node, ushort cmd, uchar *pdata, uchar sdata,
                uchar *presp, int *sresp, uchar *pcc, char fdebugcmd)
{
   int rc, i, n;
   uchar mycmd;
   struct ipmi_rq req;
   struct ipmi_rs *rsp;

   verbose = fdebugcmd;
   for (i = 0; i < NCMDS; i++) {
       if (ipmi_cmds[i].cmdtyp == cmd) break;
   }
   if (i >= NCMDS) {
        fprintf(fperr, "ipmi_cmd_lan2: Unknown command %x\n",cmd);
        return(-1);
        }
   if (cmd >= CMDMASK) mycmd = (uchar)(cmd & CMDMASK);  /* unmask it */
   else mycmd = (uchar)cmd;

   if (intf == NULL || (intf->opened == 0)) {
        rc = ipmi_open_lan2(gnode,guser,gpswd,fdebugcmd);
        if (rc != 0) {
           if (fdebugcmd)
              fprintf(fperr, "ipmi_cmd_lan2: interface open error %d\n",rc);
           return(rc);
        }
   }
 
   /* do the command */
   memset(&req, 0, sizeof(req));
   req.msg.cmd      = mycmd;
   req.msg.netfn    = ipmi_cmds[i].netfn;
   req.msg.lun      = ipmi_cmds[i].lun;
   req.msg.target_cmd = ipmi_cmds[i].sa;
   req.msg.data     = pdata;
   req.msg.data_len = sdata;

   rsp = intf->sendrecv(intf, &req);
   if (rsp == NULL) rc = -1;
   else {
      *pcc = rsp->ccode;
      rc = rsp->ccode; 
   }

   if (rc == 0) {
      /* copy data */
      if (rsp->data_len > *sresp) n = *sresp;
      else n = rsp->data_len;
      memcpy(presp,rsp->data,n);
      *sresp = n;
   } else {
      *sresp = 0;
      if ((gshutdown==0) || fdebugcmd)
             fprintf(fperr, "ipmi_cmd_lan2 error %d\n",rc);
   }
   return (rc);
}


/* misc ipmitool helper routines */

void lprintf(int level, const char * format, ...)
{
        va_list vptr;
#ifdef WIN32
        if (level > loglevel) return;
        va_start(vptr, format);
        vfprintf(stderr, format, vptr);
        va_end(vptr);
        fprintf(stderr,"\r\n");
#else
        static char logmsg[LOG_MSG_LENGTH];
        if (level > loglevel) return;
        va_start(vptr, format);
        vsnprintf(logmsg, LOG_MSG_LENGTH, format, vptr);
        va_end(vptr);
        fprintf(stderr, "%s\r\n", logmsg);
#endif
        return;
}

void lperror(int level, const char * format, ...)
{
        va_list vptr;
        if (level > loglevel) return;
        va_start(vptr, format);
        vfprintf(stderr, format, vptr); 
        va_end(vptr);
        fprintf(stderr,"\r\n");
        return;
}

int ipmi_oem_active(struct ipmi_intf * intf, const char * oemtype)
{
#ifdef NEEDED
        if (intf->oem == NULL) return 0;
        if (strncmp(intf->oem->name, oemtype, strlen(oemtype)) == 0)
                return 1;
#endif
        return 0;
}
#endif

/* end ipmilanplus.c */
